package net.sourceforge.chopchophttpclient.standalone;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;

import net.sourceforge.chopchophttpclient.Client;
import net.sourceforge.chopchophttpclient.DownloadResult;
import net.sourceforge.chopchophttpclient.proxy.EnvironmentProxySelector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;



public class ChopchopHTTPClient {
	
	private static String TWO_COL_80_FORMAT = "%-39.39s %-40.40s%n";
	
	private static final File CWD = new File(System.getProperty("user.dir"));

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//We must setup logging for HttpClient, otherwise it will get 16x slower due to it wasting time
		//	trying to log and catching errors internally  [rbeede, 2008-11]
		Logger.getLogger("org.apache.commons.httpclient").setLevel(Level.DEBUG);
		Logger.getLogger("httpclient.wire").setLevel(Level.OFF);  //really slow!

		
		final CmdLineOptions cmdLineOptions = new CmdLineOptions();
		
		final CmdLineParser parser = new CmdLineParser(cmdLineOptions);
		try {
			parser.parseArgument(args);
		} catch (CmdLineException e) {
			System.err.println(e.getMessage());
			System.exit(255);
		}
		
		
		final URI[] uris;
		try {
			uris = cmdLineOptions.getURIs();
		} catch (URISyntaxException e) {
			System.err.println(e.getMessage());
			System.exit(254);
			return;
		}
		
		
		System.out.println("Current working directory:  " + CWD.getAbsolutePath());
		System.out.println();
		
		
		//Header for list
		System.out.printf(TWO_COL_80_FORMAT, "URI", "DESTINATION FILE");
		System.out.println("======================================= ========================================");
		System.out.println();
		
		//Create list of target file names
		final File[] targetFiles = new File[uris.length];
		for(int i = 0; i < uris.length; i++) {
			final URI currURI = uris[i];
			
			targetFiles[i] = generateUniqueFilename(currURI, targetFiles);
			
			System.out.printf(TWO_COL_80_FORMAT, currURI, targetFiles[i].getAbsolutePath());
		}
		
		System.out.println();
		
		System.out.println("Commencing download of " + uris.length + " URIs...");
		
		
		final Client client = new Client(cmdLineOptions.getHttpUsername(), cmdLineOptions.getHttpPassword(), new EnvironmentProxySelector(), null);
		
		
		final DownloadResult[] downloadResults = client.download(uris, targetFiles, null);
		
		
		System.out.println("Finished downloading");
		System.out.println();
		
		for(DownloadResult result : downloadResults) {
			System.out.printf(TWO_COL_80_FORMAT,
					result.getURI(),
					result.getServerResponseString() + "\t" + result.getDownloadTimeSeconds() + " seconds"
					);
		}
		
		System.out.println();
		System.out.println("All done");
	}
	
	
	/**
	 * Uses current working directory 
	 * 
	 * @param uri
	 * @param alreadyUsed Do <b>NOT</b> resort this array or it messes up the external one passed in
	 * @return
	 */
	private static File generateUniqueFilename(final URI uri, final File[] alreadyUsed) {
		File candidateFile = null;
		
		final String uriPath = uri.getPath();

		while(null == candidateFile || candidateFile.exists()) {
			if(null != uriPath && uriPath.lastIndexOf('/') != -1) {
				String candidateName = uriPath.substring(uriPath.lastIndexOf('/')).trim();
				if("".equals(candidateName) || "/".equals(candidateName)) {
					//Use the host name
					candidateFile = new File(CWD, uri.getHost());
				} else {
					candidateFile = new File(CWD, candidateName);
				}
			} else {
				candidateFile = new File(CWD, uri.getHost());
			}

			if(candidateFile.exists()) {
				//Append a random number to try to make it unique
				candidateFile = new File(CWD, candidateFile.getName() + Double.toHexString(Math.random()));
			}
			
			for(File file : alreadyUsed) {
				if(candidateFile.equals(file)) {  //"file" could be null
					//Append a random number to try to make it unique
					candidateFile = new File(CWD, candidateFile.getName() + Double.toHexString(Math.random()));
					break;
				}
			}
		}

		return candidateFile;
	}
}
